
#include "decs.h"
#include <fftw3.h>

#define NCALC	4

void potsolv(REAL(*dens)[NY + 4], REAL(*psi)[NY + 4])
{
	double kx, kyp, kxp, K, Ksq, dvx, dvy, em, kern;
	REAL(*rho_tmp)[NY + 4];
	REAL delt;
	int i, j, ip, jp, npp, ij;
	int my_nint(double dum);
	int Nx = NX;
	int Ny = NY;
	static int ncall = 0;
	static double Ksqmax, Kmax;

	static double *rho, *pot;
	static fftw_complex *krho, *kpot;
	static fftw_plan p, pinv;
	FILE *wf;

	void unroll(REAL(*var)[NY + 4], REAL delt);
	void bound_var(REAL(*var)[NY + 4], double t);

	static int first_call = 1;

	/* set up arrays */
	if (first_call == 1) {
		first_call = 0;

		rho =
		    (double *) malloc(NX * 2 * (NY / 2 + 1) *
				      sizeof(double));
		krho =
		    (fftw_complex *) malloc(NX * (NY / 2 + 1) *
					    sizeof(fftw_complex));
		kpot = krho;
		pot = rho;

		/* create plans, manage wisdom */
		wf = fopen("potsolv.wisdom", "r");
		if (wf != NULL) {
			fftw_import_wisdom_from_file(wf);
			fclose(wf);
		}

		p = fftw_plan_dft_r2c_2d(Nx, Ny, rho, krho, FFTW_MEASURE);
		pinv =
		    fftw_plan_dft_c2r_2d(Nx, Ny, kpot, pot, FFTW_MEASURE);

		/* Uncomment to estimate, rather than using wisdom.
		   Wisdom is a bit overrated */
		/*
		   p =    rfftw2d_create_plan(Nx, Ny, FFTW_REAL_TO_COMPLEX,
		   FFTW_ESTIMATE ) ;
		   pinv = rfftw2d_create_plan(Nx, Ny, FFTW_COMPLEX_TO_REAL,
		   FFTW_ESTIMATE ) ;
		 */

		wf = fopen("potsolv.wisdom", "w");
		if (wf == NULL) {
			fprintf(stderr,
				"Cannot export wisdom to potsolv.wisdom\n");
			exit(1);
		}
		fftw_export_wisdom_to_file(wf);
		fclose(wf);

		/* this is the outer boundary of the disk in K-space
		   where the potential is calculated */
		if (dx > dy)
			Ksqmax = 0.5 * M_PI * M_PI / (dx * dx);
		else
			Ksqmax = 0.5 * M_PI * M_PI / (dy * dy);
		Kmax = sqrt(Ksqmax);
	}

	/* move density to working array */
	rho_tmp = work1;
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			rho_tmp[i][j] = dens[i][j];

	/* unroll density field to nearest periodic point */
	npp = my_nint(t * q * W * Lx / Ly);
	delt = t - npp * Ly / (q * W * Lx + SMALL);
	unroll(rho_tmp, delt);

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			rho[i * NY + j] = rho_tmp[i][j];

	/* forward fft */
	fftw_execute(p);

	/* this bit calculates the gravitational, hydro shear stress */
	if (ncall % NCALC == 0) {
		alphag = 0.;
		alphah = 0.;

		/* calculate contribution to alpha from gravity */
		for (i = 0; i < NX; i++) {
			ip = (i + NX / 2) % NX - NX / 2;
			kxp = 2. * M_PI * ip / (NX * dx);

			for (j = 0; j < NY / 2; j++) {

				jp = (j + NY / 2) % NY - NY / 2;

				ij = i * (NY / 2 + 1) + j;

				if (ip != 0 || jp != 0) {
					kyp = 2. * M_PI * jp / (NY * dy);
					kx = kxp + q * W * kyp * delt;
					K = sqrt(kx * kx + kyp * kyp);

					/* the factor of 2 takes account of the fact that we're only 
					   integrating over half the fourier domain.  It's not
					   quite accurate because there are still some wavenumbers
					   left out, but they make a negligible contribution to
					   alphag. */
					alphag +=
					    2. * (krho[ij][0] *
						  krho[ij][0] +
						  krho[ij][1] *
						  krho[ij][1]) * G * kx *
					    kyp * M_PI / (K * K * K * NX *
							  NX * NY * NY);
				}
			}
		}

		/* and from Reynolds stress */
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++) {
				dvx = 0.5 * (vx[i][j] + vx[i + 1][j]);
				dvy = 0.5 * (vy[i][j] + vy[i][j + 1]);
				alphah += S[i][j] * dvx * dvy / (NX * NY);
			}

		em = 0.;
		for (i = 0; i < NX; i++)
			for (j = 0; j < NY; j++)
				em += e[i][j];
		em /= NX * NY;

		alphag /= g2 * (g2 - 1.) * em * q;
		alphah /= g2 * (g2 - 1.) * em * q;
	}
	ncall++;

	/* solve Poisson's equation */
	for (i = 0; i < NX; i++) {
		ip = (i + NX / 2) % NX - NX / 2;
		kxp = 2. * M_PI * ip / (NX * dx);

		for (j = 0; j < NY / 2 + 1; j++) {
			jp = (j + NY / 2) % NY - NY / 2;

			kyp = 2. * M_PI * jp / (NY * dy);
			kx = kxp + q * W * kyp * delt;
			Ksq = kx * kx + kyp * kyp;

			ij = i * (NY / 2 + 1) + j;

			if (Ksq < Ksqmax && Ksq > 0.) {
				K = sqrt(Ksq);
				kern =
				    -2. * M_PI * G * (1. / K - 1. / Kmax);
				kpot[ij][0] = kern * krho[ij][0];
				kpot[ij][1] = kern * krho[ij][1];
			} else {
				kpot[ij][0] = 0.;
				kpot[ij][1] = 0.;
			}
		}
	}

	/* inverse fftw */
	fftw_execute(pinv);

	/* move pot into 2D array; normalize */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			psi[i][j] = (double) pot[i * NY + j] / (NX * NY);

	/* reroll potential */
	unroll(psi, -delt);

	/* apply boundary conditions */
	bound_var(psi, t);

	/* done! */
}
