
#include "decs.h"

void sweepx()
{
	double vxp, mdotp;
	REAL(*dq)[NY + 4];
	REAL(*mdot)[NY + 4];
	REAL(*px)[NY + 4], (*py)[NY + 4];
	void bound_var(REAL(*var)[NY + 4], double t);
	double slope_lim(double y1, double y2, double y3);
	int i, j;

	/* set auxiliary array names */
	px = work1;
	py = work2;
	mdot = work3;
	dq = work4;

	/* transform to momenta */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			px[i][j] =
			    0.5 * (S[i - 1][j] + S[i][j]) * vx[i][j];
			py[i][j] =
			    0.5 * (S[i][j - 1] + S[i][j]) * vy[i][j];
		}		/* ... no need to bound px,py */

	/* create dimensionless transport variable */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			vx[i][j] *= dt / dx;
	bound_var(vx, t);

	/* calculate mass flux */
	for (i = -1; i < NX; i++)
		for (j = 0; j < NY; j++)
			dq[i][j] =
			    slope_lim(S[i - 1][j], S[i][j], S[i + 1][j]);

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			mdot[i][j] = (vx[i][j] > 0.) ?
			    vx[i][j] * (S[i - 1][j] +
					(1. - vx[i][j]) * dq[i -
							     1][j]) :
			    vx[i][j] * (S[i][j] -
					(1. + vx[i][j]) * dq[i][j]);
		}
	bound_var(mdot, t);	/* this is the one flux that needs to be bounded,
				   or else we need 3 ghost zones */

	/* get energy flux */
	for (i = -1; i < NX + 1; i++)
		for (j = 0; j < NY; j++)
			dq[i][j] = slope_lim(e[i - 1][j] / S[i - 1][j],
					     e[i][j] / S[i][j],
					     e[i + 1][j] / S[i + 1][j]);

	for (i = 0; i < NX + 1; i++)
		for (j = 0; j < NY; j++) {
			fl[i][j] = (vx[i][j] > 0.) ?
			    mdot[i][j] * (e[i - 1][j] / S[i - 1][j] +
					  (1. - vx[i][j]) * dq[i -
							       1][j]) :
			    mdot[i][j] * (e[i][j] / S[i][j] -
					  (1. + vx[i][j]) * dq[i][j]);
		}

	/* step energy */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			e[i][j] += -(fl[i + 1][j] - fl[i][j]);
			if (e[i][j] < 0.)
				fprintf(stdout,
					"sweepx e < 0 %d %d %g %g\n", i, j,
					S[i][j], e[i][j]);
		}
	bound_var(e, t);

	/* step mass */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			S[i][j] += -(mdot[i + 1][j] - mdot[i][j]);
		}
	bound_var(S, t);

	/* get vx flux */
	for (i = -1; i < NX + 1; i++)
		for (j = 0; j < NY; j++)
			dq[i][j] =
			    slope_lim(vx[i - 1][j], vx[i][j],
				      vx[i + 1][j]);

	for (i = -1; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vxp = 0.5 * (vx[i + 1][j] + vx[i][j]);
			mdotp = 0.5 * (mdot[i + 1][j] + mdot[i][j]);
			fl[i][j] = (vxp > 0.) ?
			    mdotp * (vx[i][j] +
				     (1. -
				      vxp) * dq[i][j]) * (dx /
							  dt) : mdotp *
			    (vx[i + 1][j] -
			     (1. + vxp) * dq[i + 1][j]) * (dx / dt);
		}

	/* step vx */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			px[i][j] += -(fl[i][j] - fl[i - 1][j]);

	/* get vy flux */
	for (i = -1; i < NX + 1; i++)
		for (j = 0; j < NY; j++)
			dq[i][j] =
			    slope_lim(vy[i - 1][j], vy[i][j],
				      vy[i + 1][j]);

	/* step vy */
	for (i = 0; i < NX + 1; i++)
		for (j = 0; j < NY; j++) {
			mdotp = 0.5 * (mdot[i][j] + mdot[i][j - 1]);
			vxp = 0.5 * (vx[i][j] + vx[i][j - 1]);
			fl[i][j] = (vxp > 0.) ?
			    mdotp * (vy[i - 1][j] +
				     (1. - vxp) * dq[i -
						     1][j]) : mdotp *
			    (vy[i][j] - (1. + vxp) * dq[i][j]);
		}

	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++)
			py[i][j] += -(fl[i + 1][j] - fl[i][j]);

	/* return to velocities */
	for (i = 0; i < NX; i++)
		for (j = 0; j < NY; j++) {
			vx[i][j] = 2. * px[i][j] / (S[i][j] + S[i - 1][j]);
			vy[i][j] = 2. * py[i][j] / (S[i][j] + S[i][j - 1]);
		}
	bound_var(vx, t);
	bound_var(vy, t);
}

double slope_lim(double y1, double y2, double y3)
{
	double Dqm, Dqp, s;

	/* van leer slope limiter */
	/* N.B.: factor of 2 is missing because factor is
	   absorbed into expressions for fluxes above */
	Dqp = (y3 - y2);
	Dqm = (y2 - y1);
	s = Dqm * Dqp;
	if (s <= 0.)
		return 0.;
	else
		return (s / (Dqm + Dqp));

}
